/* 
 * Copyright (c) 2008-2010, Hazel Ltd. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.hazelcast.client;

import com.hazelcast.core.Transaction;

public final class ThreadContext {
    private static final ThreadLocal<ThreadContext> threadLocal = new ThreadLocal<ThreadContext>();
    private static final Object lock = new Object();
    TransactionClientProxy transactionProxy;

    public static ThreadContext get() {
        ThreadContext threadContext = threadLocal.get();
        if (threadContext == null) {
            threadContext = new ThreadContext();
            threadLocal.set(threadContext);
        }
        return threadContext;
    }

    public Transaction getTransaction(HazelcastClient client) {
        if (transactionProxy == null) {
            synchronized (lock) {
                if (transactionProxy == null) {
                    transactionProxy = new TransactionClientProxy(null, client);
                }
            }
        }
        return transactionProxy;
    }

    public void removeTransaction() {
        transactionProxy = null;
    }
}
